/* ********************************************************

   chromatic polynomial of G (Algorithm 1)

   LEDA 5.0
   g++ -I$LEDAROOT/incl -L$LEDAROOT algorithm_1.cpp -lW -lP -lG -lL -lX11 -lm

   ******************************************************** */


#include <LEDA/graph/graph.h>
#include <LEDA/core/array.h>
#include <iostream>
using namespace leda;
using namespace std;

int complete_graph(graph G);
int algorithm_1(graph &G,int col);
int edge_add(graph &G);
int edge_contraction(graph &G);

int main()
{
  graph G;
  G.read("graph.gw");

  int color;
  cin >> color;

  cout << algorithm_1(G,color) << endl;

}

int algorithm_1(graph &G,int col)
{
  int nu = G.number_of_nodes();
  int i = nu,j = 1;

  if(complete_graph(G)){
    for(;i>0;i--)
      j=j*col;
    return j;
  }
  else {
    graph G_1 = G;
    graph G_2 = G;
    node v;
    edge_add(G_1);
    edge_contraction(G_2);
    int p_1 = algorithm_1(G_1,col);
    int p_2 = algorithm_1(G_2,col);
    return p_1+p_2;
  }
}

int complete_graph(graph G)
{
  node v;
  int i=G.number_of_nodes();
  int j;

  forall_nodes(v,G){
    j=G.degree(v);
    if(i!=j+1)
      return 0;
  }

  return 1;
}

int edge_add(graph &G)
{

  G.make_undirected();
  node v,w;

  node_array<int> node_ord(G);
  forall_nodes(v,G)
    node_ord[v]=0;

  int v_num = G.number_of_nodes();

  node v1;
  for(;;){
    v1 = G.choose_node();
    if(G.degree(v1) < v_num-1)
      break;
  }
  node_ord[v1]=1;
  edge e;
  forall_adj_edges(e,v1)
    node_ord[G.opposite(v1,e)]=1;

  forall_nodes(v,G){
    if(node_ord[v]==0){
      G.new_edge(v1,v);
      break;
    }
  }
}

int edge_contraction(graph &G)
{

  G.make_undirected();
  edge e = G.choose_edge();

  node v = G.source(e);
  node w = G.opposite(v,e);

  int i = G.degree(v);
  int j = G.degree(w);

  if(i > j){
    node z = v;
    int k = i;
    v = w; i = j;
    w = z; j = k;
  }

  int G_num = G.number_of_nodes();

  if(j < G_num-1){
    node x,y;
    int check;
    edge e1,e2;
    forall_adj_edges(e1,v){
      x = G.opposite(v,e1);
      check = 0;
      if(x == w)
	check =1;
      forall_adj_edges(e2,w){
	y = G.opposite(w,e2);
	if(x == y){
	  check = 1;
	  break;
	}
      }
      if(check == 0)
	G.new_edge(x,w);
    }
  }
  G.del_node(v);
}
